function adaptFigurePosition(hCurrentFig, parentFigID)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% The current figure (defined by figure handle hCurrentFig) is positioned in relation to a parent figure.
% The parent figure (defined by parentFigID) can be identified by both a handle or a figure tag.
% If the function is called without a parentFigID, the GridCAT - main figure is used as parent figure.

if nargin < 2
    
    % no parentFigID provided, so use position of parent gridcat main figure
    hParentFig = findall(0, 'Type', 'figure', 'Tag', 'GridCAT - main figure');
    
elseif ~ishandle(parentFigID)
    
    % parentFigID seems not to be a handle, but a figure tag
    hParentFig = findall(0, 'Type', 'figure', 'Tag', parentFigID);
    
else
    % parentFigID seems to be a handle
    hParentFig = parentFigID;
        
end

if ~isempty(hParentFig)
    
    % get parent figure position and units
    parentFigPos = get(hParentFig, 'Position');
    parentFigUnits = get(hParentFig, 'Units');
    
    % get units of current figure
    currentFigUnits = get(hCurrentFig, 'Units');    
    
    % change current figure units so that they identical to those of parent figure
    set(hCurrentFig, 'Units', parentFigUnits);
    
    % get current figure position and units
    currentFigPos = get(hCurrentFig, 'Position');
    
    % change position of current figure, only if currentFig is not within parentFig range anyways
    if ~(currentFigPos(1) >= parentFigPos(1) && currentFigPos(1) <= parentFigPos(3))
    
        % define new position
        currentFig_newPos = currentFigPos;
        currentFig_newPos(1) = parentFigPos(1) + ((parentFigPos(3)-currentFigPos(3))/2);

        % set new position in appropriate units    
        set(hCurrentFig, 'Position', currentFig_newPos);

        % change units of current figure back to its original units
        set(hCurrentFig, 'Units', currentFigUnits, 'Visible', 'on');
    end
    
end

drawnow