function [oriPerVoxel_rad, amplitudePerVoxel] = betas2ori(xFoldSymmetry, betaVol_sin, betaVol_cos)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
% 

% calculate grid orientation per voxel
oriPerVoxel_rad = atan2(betaVol_sin, betaVol_cos);

% transform grid ori from range [-pi pi] to [0 2*pi]
oriPerVoxel_rad(oriPerVoxel_rad<0)=oriPerVoxel_rad(oriPerVoxel_rad<0)+(2*pi);

% Voxel-wise orientations are divided by x due to x-fold symmetry,
% in order bring back grid ori (in radians) from range [0 360] to [0 360/x] degrees.
oriPerVoxel_rad = oriPerVoxel_rad/xFoldSymmetry;

% calculate the amplitude per voxel (might serve e.g. for weighting of individual voxels later)
amplitudePerVoxel =  sqrt((betaVol_cos.*betaVol_cos) + (betaVol_sin.*betaVol_sin));
