function meanGridOri_rad = calcMeanGridOri(xFoldSymmetry, maskFile, GLM1_results, useWeightingForVoxels)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function returns the mean grid orientation within a range of 0 and (360/xFoldSymmetry) degrees in [rad]

% read mask image
[~, maskVol] = loadImage_SPM(maskFile); 

% get orientation and amplitude per voxel
oriPerVoxelWithinROI_rad = GLM1_results.oriPerVoxel_rad(maskVol>0);
amplitudePerVoxelWithinROI = GLM1_results.amplitudePerVoxel(maskVol>0);

% For some voxels, the estimated orientation might be NaN, because of NaN's in the beta images of GLM1.
% These NaN voxels are discarded.
oriPerVoxelWithinROI_rad(isnan(oriPerVoxelWithinROI_rad)) = [];
amplitudePerVoxelWithinROI(isnan(amplitudePerVoxelWithinROI)) = [];

% if specific weighting of individual voxels is not needed, all voxels are weighted with 1
if ~useWeightingForVoxels
    amplitudePerVoxelWithinROI = ones(length(amplitudePerVoxelWithinROI),1);
end

% Transform voxelwise orientations from range [0 360/xFoldSymmetry] to [0 360] degrees in radians,
% in order to enable calculation of subsequent trigonometric functions which expect this range.
oriPerVoxelWithinROI_rad = oriPerVoxelWithinROI_rad * xFoldSymmetry;

% Transformation from polar to cartesian cordinate system (for weighting)
[X,Y] = pol2cart(oriPerVoxelWithinROI_rad, amplitudePerVoxelWithinROI); 

% find the mean across X and across Y separately
mean_X = nansum(X)/length(oriPerVoxelWithinROI_rad);
mean_Y = nansum(Y)/length(oriPerVoxelWithinROI_rad);

% Transformation back from cartesian to polar cordinate system
[meanGridOri_rad, meanAmplitude] = cart2pol(mean_X, mean_Y);

% info: meanAmplitude is not used or saved at the moment. maybe needed for future purposes.

% transform mean grid ori from range [-pi pi] to [0 2*pi]
if meanGridOri_rad < 0
    meanGridOri_rad = meanGridOri_rad + (2*pi);
end

meanGridOri_rad = meanGridOri_rad / xFoldSymmetry;
                    

