function changeFontSizesIfReq(handles, mainFigureHandle)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% The same font size might appear bigger or smaller on a platform (e.g. Mac)
% as compared to other platforms (e.g. Unix or Windows).
% To overcome this problem, this function changes all font sizes of the GUI
% depending on the used platform.

% SETTINGS:
changeFactor = 1.3; % applied to mac os systems

if ismac() % alternatively: ispc() or isunix()

    fieldRecord = fieldnames(handles);

    for elemIdx = 1:length(fieldRecord)

        if isprop(getfield(handles,fieldRecord{elemIdx}), 'FontSize')
            set(getfield(handles,fieldRecord{elemIdx}),'FontSize',get(getfield(handles,fieldRecord{elemIdx}),'FontSize')*changeFactor);
        end
    end
    
    % save info about changed font size on GUI user data, so that it will not be changed another time
    ud = getUserData(mainFigureHandle, 'UserData');
    ud.fontSizeAdapted = 1;
    set(mainFigureHandle, 'UserData', ud);
    
end

drawnow
