function ud = getUserData(h, arg)

% This function loads the UserData of a handle h, similar to get(h, 'UserData').
% However,
% on some but not all operating systems (e.g. Windows10), the UserData of a
% figure is defined as an empty GRAPHICS array (class matlab.graphics.Graphics),
% while on other platforms, the UserData is just an empty variable.
% As using the class might be problematic, an empty UserData is converted to an
% empty variable here.

if nargin > 1
    if ~strcmp(arg, 'UserData')
        warning('Function getUserData() should only be used for loading the UserData and not other fields of a handle h.');
    end
end

ud = get(h, 'UserData');

if isempty(ud) % returns 1 for both normal variables and an empty graphics array
    ud = [];
end


