function outputData = gridMetric_gridCodeResponseMagnitude(dispOutput_flag, mask_images, modelDir, conNrOfInterest)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function calculates the grid code response magnitude based on one or multiple contrasts.
% If conNrOfInterest is not defined at function call, all contrasts in SPM.mat will be displayed.

% prepare output string
outputTxt = {};

% read model data from SPM.mat
SPMmatFileInfo = load([modelDir filesep 'SPM.mat']);

% Show results for all contrasts in SPM.mat, if no contrast numbers of interest are specified
if ~exist('conNrOfInterest', 'var')
    conNrOfInterest = 1:length(SPMmatFileInfo.SPM.xCon);
end

for maskIdx = 1:length(mask_images)
    
    [~, mask_fnam, mask_fext] = fileparts(mask_images{maskIdx});
    outputTxt{end+1} = repmat('-',1,100);    
    outputTxt{end+1} = sprintf('ROI #%i: %s', maskIdx, [mask_fnam mask_fext]);       

    % read mask image
    [~, roi_vol] = loadImage_SPM(mask_images{maskIdx});

    for conIdx = conNrOfInterest

        outputTxt{end+1} = ' ';
        outputTxt{end+1} = sprintf('CONTRAST #%i: %s', conIdx, SPMmatFileInfo.SPM.xCon(conIdx).name);
        
        % read con- and t-images
        [~, conImg_vol] = loadImage_SPM([modelDir filesep sprintf('con_%04i.nii',conIdx)]);
        % [~, tStatImg_vol] = loadImage_SPM([modelDir filesep sprintf('spmT_%04i.nii',conIdx)]);

        % get voxel-wise values from within the ROI-mask
        voxelsWithinMask_conValues = conImg_vol(roi_vol>0);
        % voxelsWithinMask_tValues = tStatImg_vol(roi_vol>0);

        % average across single voxels and output the results
        outputTxt{end+1} = sprintf('   magnitude of grid code response (mean con-value within ROI): %.3f ', nanmean(voxelsWithinMask_conValues));
        % outputTxt{end+1} = sprintf('   mean t-Value within mask: %.5f', nanmean(voxelsWithinMask_tValues));
        
        % get filename of current mask
        [~, fnam, fext] = fileparts(mask_images{maskIdx});
        
        % write data to output structure
        outputData.mask(maskIdx).maskFile = [fnam fext];
        outputData.mask(maskIdx).contrast(conIdx).contrastName = SPMmatFileInfo.SPM.xCon(conIdx).name;
        outputData.mask(maskIdx).contrast(conIdx).meanConValueWithinMask = nanmean(voxelsWithinMask_conValues);
        % outputData.mask(maskIdx).contrast(conIdx).meanTValueWithinMask = nanmean(voxelsWithinMask_tValues);
        outputData.mask(maskIdx).contrast(conIdx).nVoxelsWithinMask = length(voxelsWithinMask_conValues);
        outputData.mask(maskIdx).contrast(conIdx).nNanVoxelsWithinMask = sum(isnan(voxelsWithinMask_conValues));
        
    end
    
    outputTxt{end+1} = ' ';
    
end

if dispOutput_flag
    % display results in output GUI
    output_GUI(outputTxt');
end





