function outputData = gridMetric_meanGridOriWithinROI(dispOutput_flag, mask_images, GLM1_results)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function calculates the mean grid orientation across voxels within an ROI.

% prepare output string
outputTxt = {};

% perform all calculations separately for each region of interest (mask)
for maskIdx = 1:length(mask_images)
    
    curMaskName = mask_images{maskIdx};
    [~, mask_fnam, mask_fext] = fileparts(curMaskName);

    outputTxt{end+1} = ' ';
    outputTxt{end+1} = repmat('-',1,100);    
    outputTxt{end+1} = sprintf('ROI #%i: %s', maskIdx, [mask_fnam mask_fext]);
    
    % write data to output structure
    [~, fname, fext] = fileparts(curMaskName);
    outputData.mask(maskIdx).maskFile = [fname fext];
    
    % calculate meanGridOri separately for each event-type
    for gridEventTypeIdx = 1:length(GLM1_results.gridEventType)
        
        curEventName = GLM1_results.gridEventType(gridEventTypeIdx).eventName;
        
        % write data to output structure
        outputData.mask(maskIdx).gridEventType(gridEventTypeIdx).gridEventName = curEventName;
        
        % calculate meanGridOri separately for each run
        for runIdx = 1:length(GLM1_results.gridEventType(1).run)
            
            meanOri_weighted_rad = calcMeanGridOri(GLM1_results.cfg.GLM.xFoldSymmetry, curMaskName, GLM1_results.gridEventType(gridEventTypeIdx).run(runIdx), 1);
            meanOri_notWeighted_rad = calcMeanGridOri(GLM1_results.cfg.GLM.xFoldSymmetry, curMaskName, GLM1_results.gridEventType(gridEventTypeIdx).run(runIdx), 0);
            
            outputTxt{end+1} = ' ';
            outputTxt{end+1} = sprintf('GRID EVENT ''%s'' - RUN %i:', curEventName, runIdx);
            outputTxt{end+1} = sprintf('   mean grid orientation = %.3f degrees (with amplitude-based weighting of voxels)', meanOri_weighted_rad*180/pi);
            outputTxt{end+1} = sprintf('   mean grid orientation = %.3f degrees (without amplitude-based weighting of voxels)', meanOri_notWeighted_rad*180/pi);
            
            % write data to output structure
            outputData.mask(maskIdx).gridEventType(gridEventTypeIdx).run(runIdx).meanGridOri_weighted_rad = meanOri_weighted_rad;
            outputData.mask(maskIdx).gridEventType(gridEventTypeIdx).run(runIdx).meanGridOri_notWeighted_rad = meanOri_notWeighted_rad;
        end
        
        % calculate meanGridOri also averaged across all runs
        meanOri_weighted_rad = calcMeanGridOri(GLM1_results.cfg.GLM.xFoldSymmetry, curMaskName, GLM1_results.gridEventType(gridEventTypeIdx).allRunsAvg, 1);
        meanOri_notWeighted_rad = calcMeanGridOri(GLM1_results.cfg.GLM.xFoldSymmetry, curMaskName, GLM1_results.gridEventType(gridEventTypeIdx).allRunsAvg, 0);        

        outputTxt{end+1} = ' ';
        outputTxt{end+1} = sprintf('GRID EVENT ''%s'' - ALL RUNS AVERAGE:', curEventName);
        outputTxt{end+1} = sprintf('   mean grid orientation = %.3f degrees (with amplitude-based weighting of voxels)', meanOri_weighted_rad*180/pi);
        outputTxt{end+1} = sprintf('   mean grid orientation = %.3f degrees (without amplitude-based weighting of voxels)', meanOri_notWeighted_rad*180/pi);
        
        % write data to output structure
        outputData.mask(maskIdx).gridEventType(gridEventTypeIdx).allRunsAvg.meanGridOri_weighted_rad = meanOri_weighted_rad;
        outputData.mask(maskIdx).gridEventType(gridEventTypeIdx).allRunsAvg.meanGridOri_notWeighted_rad = meanOri_notWeighted_rad;
        
    end
end

if dispOutput_flag
    % display results in output GUI
    output_GUI(outputTxt');
end