function loadDesignParameters(handles)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% Restore all settings and specifications of the GridCAT GUI from a design parameters file
% Design parameters can be stored manually by the user or are stored automatically
% (within GridCAT_GLMx.mat) when a GLM is carried out.

% open file selection dialog to get design parameters file
matFile = selectionDlg_GUI({'getFile'}, 'Select design parameters file', pwd, {'*'}, 0, [1 1], 'off', 0, 'GridCAT - main figure', {'.mat'}, 'on');

% if file has been selected
if ~isempty(matFile)

    % load mat-file that contains design parameters
    matFileData = load(matFile{1});

    % set GUI properties to reconstruct previous status
    fieldRecord = matFileData.cfg.GUIdata.fieldRecord;
    element = matFileData.cfg.GUIdata.element;
    for elemIdx = 1:length(fieldRecord)
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'String')
            set(getfield(handles,fieldRecord{elemIdx}),'String',element(elemIdx).String);
        end
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'Value')
            set(getfield(handles,fieldRecord{elemIdx}),'Value',element(elemIdx).Value);
        end
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'Enable')
            set(getfield(handles,fieldRecord{elemIdx}),'Enable',element(elemIdx).Enable);
        end
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'Visible')
            set(getfield(handles,fieldRecord{elemIdx}),'Visible',element(elemIdx).Visible);
        end
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'UserData')
            set(getfield(handles,fieldRecord{elemIdx}),'UserData',element(elemIdx).UserData);
        end 
        if isprop(getfield(handles,fieldRecord{elemIdx}), 'Data')
            set(getfield(handles,fieldRecord{elemIdx}),'Data',element(elemIdx).Data);
        end     
    end
end

drawnow