function varargout = selectionDlg_GUI(varargin)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This dialog can be used instead of the standard (OS-dependent) functions uiputfile(), uigetfile(), and uigetdir()
%
% Call this function with the following parameters:
% output = selectionDlg_GUI({'type'}, title, path, filter, alwaysShowDir_flag, nParallelSelecetions_MinMax, visibleNewDirButton, ignoreHDRfiles, parentFigTag, allowedExtensions, showAllowedExtensions, defaultFilename)
% 
% Parameters:
%	type: {'putFile'}, {'getFile'}, {'getDir'}
%	title: 'Title of the dialog';
%	path: e.g. pwd, '/Users/matthias/Desktop/data'
%	filter: files and directories are just shown if they contain one of the strings in filter, e.g., {'*'}, {'*.txt', '*.xls'}, {'asdf', 'xyz'} 
%	alwaysShowDir_flag: 1 or 0. Initial checkbox-value that specifies, whether directories are always shown, even if their name does not contain any string in "filter".
%	nParallelSelecetions_MinMax: Min/Max nr. of lines/items that can be selected in parallel. Use inf for no max-limitation. Examples: [2 2], [2 10], [2 inf]
%	visibleNewDirButton: Use 'on' or 'off' to define whether a button is shown to create a new directory
%   ignoreHDRfiles: 1 or 0. If 1, files with the extension .hdr are not displayed by the file selection dialog
%	parentFigTag: figure tag of parent figure, on which selected paths are stored, in oder to recall them later for the "previous paths" popupmenu. Use a valid figure tag of a figure, e.g. 'hello figure tag'. an invalid or dummy tag (e.g., '') will produce an empty list.
%	allowedExtensions: Only needed for type "getfile" or "putfile". Example: {'.txt', '.xls'}, use {'*'} if all extensions are allowed
%	showAllowedExtensions: Only needed for type "getfile" or "putfile". Use 'on' or 'off' to define whether the allowed file-extensions are shown
%	defaultFilename: Only needed for type "putfile".  Default filename for "putfile" dialog. Example: 'default_filename.txt'
%
% e.g.
%   output = selectionDlg_GUI({'putFile'}, 'Put file to...', pwd, {'*'}, 0, [1 1], 'on', 1, 'GridCAT - main figure', {'.txt', '.nii'}, 'on', 'temp_test.txt')
%   output = selectionDlg_GUI({'getFile'}, 'Select files...', pwd, {'*'}, 0, [1 inf], 'off', 1, 'GridCAT - main figure', {'.txt', '.nii'}, 'on')
%   output = selectionDlg_GUI({'getDir'}, 'Select directory...', pwd, {'*'}, 0, [1 1], 'on', 1, 'GridCAT - main figure')

% SELECTIONDLG_GUI MATLAB code for selectionDlg_GUI.fig
%      SELECTIONDLG_GUI, by itself, creates a new SELECTIONDLG_GUI or raises the existing
%      singleton*.
%
%      H = SELECTIONDLG_GUI returns the handle to a new SELECTIONDLG_GUI or the handle to
%      the existing singleton*.
%
%      SELECTIONDLG_GUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in SELECTIONDLG_GUI.M with the given input arguments.
%
%      SELECTIONDLG_GUI('Property','Value',...) creates a new SELECTIONDLG_GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before selectionDlg_GUI_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to selectionDlg_GUI_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help selectionDlg_GUI

% Last Modified by GUIDE v2.5 15-Oct-2016 07:07:21

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @selectionDlg_GUI_OpeningFcn, ...
                   'gui_OutputFcn',  @selectionDlg_GUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before selectionDlg_GUI is made visible.
function selectionDlg_GUI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to selectionDlg_GUI (see VARARGIN)

% Choose default command line output for selectionDlg_GUI
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% ------------------------------------------------
% mst-edit:

% Change font size of all GUI elements depending on current platform, if necessary 
ud = getUserData(hObject, 'UserData');
if ~isfield(ud, 'fontSizeAdapted') || ~ud.fontSizeAdapted
    changeFontSizesIfReq(handles, hObject);
end

% Make the GUI modal
set(handles.figure1,'WindowStyle', 'modal');

% change figure name
set(handles.figure1, 'Name', 'GridCAT - Selection Dialog');

% give figure1 a specific tag:
set(handles.figure1, 'Tag', 'GridCAT - SelectionDlg');
   
% get varargin input
type = cell2mat(varargin{1});
title = varargin{2};
path = varargin{3};
filter = varargin{4};
alwaysShowDir_flag = varargin{5};
nParallelSelections_MinMax = varargin{6};
showNewDirButton = varargin{7};
ignoreHDRfiles_flag = varargin{8};
parentFigTag = varargin{9};
if strcmp(type, 'putFile') || strcmp(type, 'getFile')
    allowedExtensions = varargin{10}; % only needed if type is "putFile" or "getFile"
    showAllowedExtensions = varargin{11}; % only needed if type is "putFile" or "getFile"
end
if strcmp(type, 'putFile')
    defaultFilename = varargin{12}; % only needed if type is "putFile"
end

% set title
set(handles.text_dlgTitle, 'String', title);

% set listbox
ud_listbox.ignoreHDRfiles_flag = ignoreHDRfiles_flag;
ud_listbox.currentPath = path;
ud_listbox.nParallelSelections_MinMax = nParallelSelections_MinMax;
ud_listbox.lastSelection = -1; % default init
ud_listbox.lastSelectionTime = clock;
set(handles.listbox_filelist, 'UserData', ud_listbox);

% % specify listbox properties (undocumented matlab)
% jScrollPane = findjobj(handles.listbox_filelist); % function findjobj downloaded from matlabcentral
% jListbox = jScrollPane.getViewport.getView;
% jListbox.setFixedCellHeight(20)  % icon height=16 + 4px margin
% jListbox.repaint  % refresh the display

% should the "always show directories" checkbox be on or off initially?
set(handles.checkbox_showDirectories, 'Value', alwaysShowDir_flag);
ud_fig1.alwaysShowDir_defaultValue = alwaysShowDir_flag;

% set filename filter
ud_edit_filter.defaultFilter = filter;
set(handles.edit_filenameFilter, 'UserData', ud_edit_filter);
setFilenameFilter(handles, filter);

% set visibility of "new directory" button
set(handles.pushbutton_createNewDir, 'Visible', showNewDirButton);

% if type of dlg is getfile or getdir, hide "save as filename..." text and editbox
if strcmp(type, 'getFile') || strcmp(type, 'getDir')
    set(handles.text_saveAsFilename, 'Visible', 'off');
    set(handles.edit_saveAsFilename, 'Visible', 'off');
end

% if type of dlg is putFile, set default filename for "save to file" 
if strcmp(type, 'putFile')
    set(handles.edit_saveAsFilename, 'String', defaultFilename);
    ud_fig1.defaultFilename = defaultFilename;
end

% if type of dlg is putFile or getFile, just specific file extensions are allowed to select 
if strcmp(type, 'putFile') || strcmp(type, 'getFile')
    ud_fig1.allowedExtensions = allowedExtensions;
    set(handles.text_allowedExtensions, 'String', ['Valid extensions: ' createAllowedExtensionsVisStr(ud_fig1.allowedExtensions)]);
    set(handles.text_allowedExtensions, 'Visible', showAllowedExtensions);
else
    ud_fig1.allowedExtensions = '';
    set(handles.text_allowedExtensions, 'Visible', 'off');
end

% set "previous path" popupmenu
ud_fig1.parentFigTag = parentFigTag;
hParentFig = findall(0, 'Type', 'figure', 'Tag', ud_fig1.parentFigTag);
if ~isempty(hParentFig)
    ud_parentFig = getUserData(hParentFig, 'UserData');
    if isfield(ud_parentFig, 'selectionDlg_previousPaths') && ~isempty(ud_parentFig.selectionDlg_previousPaths)

        % ----------------------------------------
        % create list of previously selected paths
        
        showFullPathOfRecentPlaces = 1; % 1...full path of recent places is shown; 0...only the parent directory of recent places is shown
        
        
        if showFullPathOfRecentPlaces
            % full path of recent places is shown
            prevPaths_visList = ud_parentFig.selectionDlg_previousPaths;
            
        else
            % only the parent directory of recent places is shown
            prevPaths_visList = ud_parentFig.selectionDlg_previousPaths;
            for pathIdx = 1:length(prevPaths_visList)
                curPathStr = prevPaths_visList{pathIdx};    
                filesepPosList = find(curPathStr == filesep);
                if length(filesepPosList)>1
                    curPathStr = curPathStr(filesepPosList(end-1)+1:filesepPosList(end)-1);
                else
                    curPathStr = curPathStr(1:filesepPosList(end));
                end
                prevPaths_visList{pathIdx} = curPathStr;
            end
        end
        
        % shorten text if too long?
        shortenTextIfTooLong =  true; % show preceding ... if one or more lines of the popupmenu are too long
        if shortenTextIfTooLong
            pixelBuffer = 60; % setting
            for pathIdx = 1:length(prevPaths_visList)
                set(handles.popupmenu_previousPaths, 'String', ' ');
                visStr = prevPaths_visList{pathIdx};
                set(handles.popupmenu_previousPaths, 'String', visStr);
                popupmenuPos = get(handles.popupmenu_previousPaths, 'Position');
                visStrExtent = get(handles.popupmenu_previousPaths, 'Extent');
                while visStrExtent(3)+pixelBuffer > popupmenuPos(3)
                    visStr = visStr(2:end);
                    set(handles.popupmenu_previousPaths, 'String', ['...' visStr]);
                    visStrExtent = get(handles.popupmenu_previousPaths, 'Extent');
                    prevPaths_visList{pathIdx} = ['...' visStr];
                end
            end
        end

        
        % ----------------------------------------

        % display previous destination folders in popupmenu
        set(handles.popupmenu_previousPaths, 'String', prevPaths_visList)
        
        % save complete list of previous paths on popupmenu user data
        ud_popupmenu.previousPaths = ud_parentFig.selectionDlg_previousPaths;
        set(handles.popupmenu_previousPaths, 'UserData', ud_popupmenu);
        
    end
end

% save dlg type and additional information on user data of the main figure
ud_fig1.dlgType = type;
ud_fig1.startPath = path;
ud_fig1.output = [];
set(handles.figure1, 'UserData', ud_fig1);

% adapt position of the new GUI
adaptFigurePosition(hObject);

drawnow

% ------------------------------------------------

% UIWAIT makes selectionDlg_GUI wait for user response (see UIRESUME)
uiwait(handles.figure1); % mst-edit




% --- Outputs from this function are returned to the command line.
function varargout = selectionDlg_GUI_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% ------------------------------------------------
% mst-edit:

ud = getUserData(handles.figure1, 'UserData');
varargout{1} = ud.output;

delete(handles.figure1);
% ------------------------------------------------


% --- Executes when user attempts to close figure1.
function figure1_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% mst-edit:
if isequal(get(hObject, 'waitstatus'), 'waiting')
    uiresume(hObject);
else
    delete(hObject);
end


% --- Executes during object creation, after setting all properties.
function listbox_filelist_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filelist (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function edit_currentPath_Callback(hObject, eventdata, handles)
% hObject    handle to edit_currentPath (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_currentPath as text
%        str2double(get(hObject,'String')) returns contents of edit_currentPath as a double

% check whether the new path (user input) is a valid path
newPath = get(handles.edit_currentPath, 'String');
if exist(newPath, 'dir');
    
    % it's a valid path!    
    % now do some fine-tuning with the path name, so that it has the appropriate format on all platforms
    if length(find(newPath==filesep))>1 && newPath(end) == filesep; % e.g., user typed C:\data\
        newPath(end) = [];
    elseif isempty(find(newPath==filesep, 1)) % e.g., user typed C: under windows
        newPath(end+1) = filesep;
    end
        
    % update listbox with new path
    ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
    updateListbox(handles, newPath, ud_edit_filter.currentFilter);
else
    
    % the entered new path is not valid, so keep the current path
    ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
    set(handles.edit_currentPath, 'String', ud_listbox.currentPath);
    disp('The entered path is not valid.');
end

drawnow


% --- Executes during object creation, after setting all properties.
function edit_currentPath_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_currentPath (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton_Cancel.
function pushbutton_Cancel_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton_Cancel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

close(handles.figure1);



function edit_filenameFilter_Callback(hObject, eventdata, handles)
% hObject    handle to edit_filenameFilter (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_filenameFilter as text
%        str2double(get(hObject,'String')) returns contents of edit_filenameFilter as a double

filter = get(handles.edit_filenameFilter, 'String');
if isempty(filter)
    filter = '*';
end
filter = strsplit(filter, {',', ' '});
setFilenameFilter(handles, filter);



% --- Executes during object creation, after setting all properties.
function edit_filenameFilter_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_filenameFilter (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton_OK.
function pushbutton_OK_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton_OK (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get data from GUI
ud_fig1 = getUserData(handles.figure1, 'UserData');
currentlySelectedLine = get(handles.listbox_filelist, 'Value');
ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
if isempty(ud_listbox.currentDirList)
    % no dir or file is currently shown in the listbox (can be, e.g., because no file/dir is in accordance with the filter)
    return
end
dirList_selectedLines = ud_listbox.currentDirList(currentlySelectedLine);

% get the current path name
currentPath = get(handles.edit_currentPath, 'String');
if currentPath(end)~=filesep
    currentPath(end+1) = filesep;
end

% check whether the defined minimum number of items has been selected
if length(dirList_selectedLines)<ud_listbox.nParallelSelections_MinMax(1)
    h = msgbox(['You have selected less than ' num2str(ud_listbox.nParallelSelections_MinMax(1)) ' items.']);
    adaptFigurePosition(h, handles.figure1);
    return;
end

% check whether not more than the defined maximum number of items has been selected
if length(dirList_selectedLines)>ud_listbox.nParallelSelections_MinMax(2)
    % should actually never happen, because other code should prevent users from selecting more than the max number of lines
    h = msgbox(['You have selected more than ' num2str(ud_listbox.nParallelSelections_MinMax(2)) ' items.']);
    adaptFigurePosition(h, handles.figure1);
    return;
end

switch ud_fig1.dlgType
    
    case 'putFile'
        
        % check whether new filename is ok
        newFilename = get(handles.edit_saveAsFilename, 'String');
        [~, ~, extension] = fileparts(newFilename);
        if isempty(newFilename)  || ~isValidDirName(newFilename) || ( ~any(strcmp(extension,ud_fig1.allowedExtensions)) && ~any(strcmp('*',ud_fig1.allowedExtensions)) )
            h = msgbox(['This is not a valid filename. Please do not use special characters and use a valid file-extension (' createAllowedExtensionsVisStr(ud_fig1.allowedExtensions) ').']);
            adaptFigurePosition(h, handles.figure1);
            return;
        end
        
        % all checks were ok, so we have a valid output:
        output{1} = [currentPath newFilename];
        
    case 'getFile'
        
        % prepare cell array for output
        output = cell(length(dirList_selectedLines),1); % preallocation for speed
        
        % check whether selected lines are ok
        for selectedLineIdx = 1:length(dirList_selectedLines)
            
            % check whether current line is not a directory but a file
            if dirList_selectedLines(selectedLineIdx).isdir
                h = msgbox(['The selected item named ''' dirList_selectedLines(selectedLineIdx).name ''' is not a valid file but a directory.']);
                adaptFigurePosition(h, handles.figure1);
                return;
            end
            
            % check whether current line has an allowed extension
            [~, ~, extension] = fileparts(dirList_selectedLines(selectedLineIdx).name);
            if ~any(strcmp(extension,ud_fig1.allowedExtensions)) && ~any(strcmp('*',ud_fig1.allowedExtensions))
                h = msgbox(['The selected item named ''' dirList_selectedLines(selectedLineIdx).name ''' does not have a valid extension. Valid extensions are: ' createAllowedExtensionsVisStr(ud_fig1.allowedExtensions)]);
                adaptFigurePosition(h, handles.figure1);
                return;
            end
            
            % all checks for the current item were ok, so we have a valid output
            output{selectedLineIdx} = [currentPath dirList_selectedLines(selectedLineIdx).name];
            
        end
        
    case 'getDir'
        
        % prepare cell array for output
        output = cell(length(dirList_selectedLines),1); % preallocation for speed
        
        % check whether selected lines are ok
        for selectedLineIdx = 1:length(dirList_selectedLines)
            
            % check whether current line is not a file but a directory
            if ~dirList_selectedLines(selectedLineIdx).isdir || strcmp(dirList_selectedLines(selectedLineIdx).name, '..')
                h = msgbox(['The selected item named ''' dirList_selectedLines(selectedLineIdx).name ''' is not a valid directory.']);
                adaptFigurePosition(h, handles.figure1);
                return;
            end
           
            % all checks for the current item were ok, so we have a valid output
            output{selectedLineIdx} = [currentPath dirList_selectedLines(selectedLineIdx).name];
            
        end          
end

% save current path on parent figure's user data, for later use in "previous paths" popupmenu
hParentFig = findall(0, 'Type', 'figure', 'Tag', ud_fig1.parentFigTag);
if ~isempty(hParentFig)
    ud_parentFig = getUserData(hParentFig, 'UserData');
    if isfield(ud_parentFig, 'selectionDlg_previousPaths') && ~isempty(ud_parentFig.selectionDlg_previousPaths)
        
        % if the current path is already in the list, delete the existing entry
        isIdenticalEntry = strcmp(ud_parentFig.selectionDlg_previousPaths, currentPath);
        ud_parentFig.selectionDlg_previousPaths(isIdenticalEntry) = [];
        
        % include new path at the beginning of already existing list
        newPrevPathList = [currentPath ud_parentFig.selectionDlg_previousPaths];
    else
        % start a new list, since there is no existing one
        newPrevPathList{1} = currentPath;
    end
    
    % delete end of list if it gets longer than the max number of entries
    maxNumberOfEntries = 10; % setting: max number of entries in the list of previous paths 
    newPrevPathList(maxNumberOfEntries+1:end) = [];
    
    ud_parentFig.selectionDlg_previousPaths = newPrevPathList;
    set(hParentFig, 'UserData', ud_parentFig);
end

% save variables for output on userdata of figure 1
ud_fig1 = getUserData(handles.figure1, 'UserData');
ud_fig1.output = output;
set(handles.figure1, 'UserData', ud_fig1);

close(handles.figure1);

drawnow


% --- Executes on selection change in listbox_filelist.
function listbox_filelist_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filelist (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns listbox_filelist contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filelist

% only allow a maximum (defined by fcn-parameter nParallelSelecetions_MinMax) number of selected lines.
% if more lines are selected, change back to previous selections.
ud = getUserData(handles.listbox_filelist, 'UserData');
previouslySelectedLine = ud.lastSelection;
lastSelectionTime = ud.lastSelectionTime;
currentlySelectedLine = get(handles.listbox_filelist, 'Value');
if numel(currentlySelectedLine) > ud.nParallelSelections_MinMax(2)    
    if (previouslySelectedLine(1) > 0) % && (previouslySelectedLine <= length(get(handles.listbox_filelist, 'String')))
        set(handles.listbox_filelist, 'Value', previouslySelectedLine);
    else
        set(handles.listbox_filelist, 'Value', 1);
    end
    
    % end function here, if too many lines were selected
    return
    
end

% write last selected line and time of last selection change on user data
ud.lastSelection = currentlySelectedLine;
ud.lastSelectionTime = clock;
set(handles.listbox_filelist, 'UserData', ud);

% check whether the user double-clicked on a directory, in order to enable navigation.
% only perform this check, if only a single line was selected and double-clicked.
% (double-click means two clicks within 1.5 sec)
if numel(currentlySelectedLine) == 1 && isequal(currentlySelectedLine, previouslySelectedLine) && etime(clock, lastSelectionTime)<=1.5
    
    % User double-clicked on line
    
    if ud.currentDirList(currentlySelectedLine).isdir
        
        % User double-clicked on a directory-line
        
        if ~strcmp(ud.currentDirList(currentlySelectedLine).name, '.')
        
            if strcmp(ud.currentDirList(currentlySelectedLine).name(1:2), '..');
                
                % User double-clicked on the '..' line to move to the parent directory

                filesepPosList = find(ud.currentPath == filesep);
                if length(filesepPosList)>1
                    % go to the parent directory
                    newPath = ud.currentPath(1:filesepPosList(end)-1);
                else
                    % go to the parent directory, which is the very first parent directory of the path
                    newPath = ud.currentPath(1:filesepPosList(end));
                end
            else
                
                % User double-clicked on a normal directorie's line to show that directories content
                
                if ud.currentPath(end) == filesep;
                    % The current directory is the very first parent directory of the path
                    newPath = [ud.currentPath ud.currentDirList(currentlySelectedLine).name];
                else
                    % The current directory is a normal directory (not the very first) of the path
                    newPath = [ud.currentPath filesep ud.currentDirList(currentlySelectedLine).name];
                end
            end
            
            % Update listbox
            ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
            updateListbox(handles, newPath, ud_edit_filter.currentFilter);
            
        end
    end
end

drawnow


% --- Executes on button press in pushbutton_restoreDefaults.
function pushbutton_restoreDefaults_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton_restoreDefaults (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get data from GUI
ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
ud_fig1 = getUserData(handles.figure1, 'UserData');

% set current path on listbox user data back to default
ud_listbox.currentPath = ud_fig1.startPath;
set(handles.listbox_filelist, 'UserData', ud_listbox);

% Update filename filter and listbox
setFilenameFilter(handles, ud_edit_filter.defaultFilter);

% if we have a default filename (as in a putfile dlg), also reset it
if isfield(ud_fig1, 'defaultFilename')
    set(handles.edit_saveAsFilename, 'String', ud_fig1.defaultFilename);
end

% was the "always show directories" checkbox be on or off initially?
set(handles.checkbox_showDirectories, 'Value', ud_fig1.alwaysShowDir_defaultValue);

drawnow


% --- Executes on button press in checkbox_showDirectories.
function checkbox_showDirectories_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox_showDirectories (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox_showDirectories

ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
updateListbox(handles, ud_listbox.currentPath, ud_edit_filter.currentFilter);


% --- Executes on button press in pushbutton_createNewDir.
function pushbutton_createNewDir_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton_createNewDir (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

newDirName = inputdlg_GUI({'Directory name:'});

% Create new directory, if the entered new name is ok.
if ~isempty(newDirName) && isValidDirName(newDirName)
    ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
    ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
    if ~exist([ud_listbox.currentPath filesep newDirName], 'dir')
        mkdir([ud_listbox.currentPath filesep newDirName]);
    else
        h = msgbox('The entered directory name already exists.');
        adaptFigurePosition(h, handles.figure1);
    end
    updateListbox(handles, ud_listbox.currentPath, ud_edit_filter.currentFilter);
else
    disp('The entered directory name is not valid.');
end


function edit_saveAsFilename_Callback(hObject, eventdata, handles)
% hObject    handle to edit_saveAsFilename (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit_saveAsFilename as text
%        str2double(get(hObject,'String')) returns contents of edit_saveAsFilename as a double


% --- Executes during object creation, after setting all properties.
function edit_saveAsFilename_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_saveAsFilename (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function updateListbox(handles, path, filter)
% update data shown in listbox

% fist of all, display an empty list (in order to indicate that the user-click has been recognized)
set(handles.listbox_filelist, 'Value', 1);
set(handles.listbox_filelist, 'String', ' ');
drawnow

ud = getUserData(handles.listbox_filelist, 'UserData');
ud.currentPath = path;

ud.currentDirList = dir(path);
ud.currentDirList = modifyDirList(ud.currentDirList, filter, get(handles.checkbox_showDirectories, 'Value'), ud.ignoreHDRfiles_flag);

dirList_str = {ud.currentDirList.name};

% get line of link to parent dir (..)
parentDirIdx = find(strcmp(dirList_str, '..'));

% -----------------------------------------------------------------
% Visualization of listbox elements (FontName: Courier)

dirList_str([ud.currentDirList.isdir]) =  strcat({'<DIR> '}, dirList_str([ud.currentDirList.isdir]));
dirList_str(~[ud.currentDirList.isdir]) = strcat({'      '}, dirList_str(~[ud.currentDirList.isdir]));

% -----------------------------------------------------------------
% 
% % ALTERNATIVELY: Visualization of listbox elements using undocumented matlab (HTML etc.)
% 
% % use HTML to visualize directories
% dirList_str([ud.currentDirList.isdir]) = strcat({['<HTML><img src="file:' ud.pathToIcons filesep 'icon_folder.gif"/>']},dirList_str([ud.currentDirList.isdir]));
% % dirList_str([ud.currentDirList.isdir]) = strcat({'<HTML><b>[DIR] '}, dirList_str([ud.currentDirList.isdir]), '</b></HTML>'); % or change color etc.: strcat({'<HTML><FONT color="red">[DIR] '}, dirList_str([ud.currentDirList.isdir]), '</FONT></HTML>');
% 
% % use HTML to visualize link to parent directory (..)
% if ~isempty(parentDirIdx)
%     dirList_str(parentDirIdx) = strcat({['<HTML><img src="file:' ud.pathToIcons filesep 'icon_upfolder.gif"/>']}, ud.currentDirList(parentDirIdx).name); % although they seem needless, {} brackets are actually necessary here
% %     dirList_str(parentDirIdx) = strcat({'<HTML><b>'}, ud.currentDirList(parentDirIdx).name, '</b></HTML>');
% end
% 
% % use HTML to visualize files
% dirList_str(~[ud.currentDirList.isdir]) = strcat({['<HTML><img src="file:' ud.pathToIcons filesep 'icon_file.gif"/>']},dirList_str(~[ud.currentDirList.isdir]));
% 
% -----------------------------------------------------------------

% set handles
set(handles.listbox_filelist, 'String', dirList_str);
set(handles.edit_currentPath, 'String', ud.currentPath);
ud.lastSelection = -1;
set(handles.listbox_filelist, 'UserData', ud);

drawnow



function setFilenameFilter(handles, filter)
% Set filter for filenames.
% Also, perform an update on the listbox content (with the new filter),
% where the current path is taken from listbox-UserData.currentPath

ud_edit_filter = getUserData(handles.edit_filenameFilter, 'UserData');
ud_edit_filter.currentFilter = filter;
set(handles.edit_filenameFilter, 'UserData', ud_edit_filter);

filter_visStr = '';
for filterIdx = 1:length(filter)
    filter_visStr = [filter_visStr filter{filterIdx} ', '];
end
filter_visStr(end-1:end) = [];
set(handles.edit_filenameFilter, 'String', filter_visStr);

ud_listbox = getUserData(handles.listbox_filelist, 'UserData');
updateListbox(handles, ud_listbox.currentPath, filter);


function visStr = createAllowedExtensionsVisStr(allowedExtList)
% create a string for visualization of allowed file extensions

visStr = '';
for extIdx = 1:length(allowedExtList);
    visStr = [visStr allowedExtList{extIdx} ', '];
end
visStr(end-1:end) = [];


function dirList_new = modifyDirList(dirList, filter, showDirectories_flag, ignoreHDRfiles_flag)
% This function modifies a list of directories/files 'dirList' (that is returned from the command 'dir').
% The function returns the same list of directories/files, but in the returned list there are the following changes:
%    All directories are shown first before files.
%    Files are only kept in the list, if they apply to one of the wildcard-filters in 'filter', e.g. {'*.m', '*.txt'}, or {*.*}
%    Directories are only kept in the list, if they apply to one of the wildcard-filters in 'filter', or if showDirectories_flag == 1
%    Files and folders that start with an '.' or '$' are removed from the list.

% sort dir-list so that directories are shown before files
[~, sortIds] = sort([dirList(1:end).isdir], 'descend');
dirList_new = dirList(sortIds);

% '..' should be always shown at the very first position if it exists
if any(strcmp({dirList_new(:).name}, '..'))
    dirList_new = [dirList_new(strcmp({dirList_new(:).name}, '..')); dirList_new(~strcmp({dirList_new(:).name}, '..'))];
end

% delete all list entries that precede with . or $ (except ..)
letters = cellfun(@(x) x(1),{dirList_new(:).name},'un',0);
badIds = find(strcmp(letters, '.') | strcmp(letters, '$'));
badIds(badIds==1) = []; % .. is in first position
dirList_new(badIds) = [];

% delete all files from list that end with .hdr or .HDR, if ignoreHDRfiles_flag is set to 1
if ignoreHDRfiles_flag
    temp_list = dirList_new;
    [temp_list(cellfun(@length, {temp_list(:).name}) < 4).name] = deal('xxxx');
    extensions = cellfun(@(x) x(end-3:end),{temp_list(:).name},'un',0);
    dirList_new(strcmpi(extensions, '.hdr') & ~[dirList_new(:).isdir]) = [];
end

% keep only entries that match with one of the filters
keepIds = [];
for filterIdx = 1:length(filter)
    keepIds = [keepIds, find(~cellfun(@isempty,regexp({dirList_new(:).name}, regexptranslate('wildcard',filter{filterIdx}))))];
end

% keep all directories, if showDirectories_flag is set to 1
if showDirectories_flag
    keepIds = [keepIds, find([dirList_new(:).isdir])];
end

% keep only entries that passed the criteria
keepIds = unique(keepIds);
dirList_new = dirList_new(sort(keepIds));


% --- Executes on selection change in popupmenu_previousPaths.
function popupmenu_previousPaths_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_previousPaths (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu_previousPaths contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_previousPaths

ud_popupmenu = getUserData(handles.popupmenu_previousPaths, 'UserData');
if isfield(ud_popupmenu, 'previousPaths') && ~isempty(ud_popupmenu.previousPaths)

    newPath = cell2mat(ud_popupmenu.previousPaths(get(handles.popupmenu_previousPaths, 'Value')));

    set(handles.edit_currentPath, 'String', newPath);
    edit_currentPath_Callback([], [], handles)
end

drawnow


% --- Executes during object creation, after setting all properties.
function popupmenu_previousPaths_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_previousPaths (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
