function spmModel_spec(cfg)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% Specify GLM using SPM functions


%% SPECIFY 1ST-LEVEL MODEL

moduleNr = 1;
for  runIdx = 1:length(cfg.rawData.run)

    matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).scans = strcat(cfg.rawData.run(runIdx).functionalScans, ',1');
    matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).cond = struct('name', {}, 'onset', {}, 'duration', {}, 'tmod', {}, 'pmod', {});
    matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).multi = {[cfg.GLM.dataDir filesep 'multipleConditions_run' num2str(runIdx) '.mat']};
    matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).regress = struct('name', {}, 'val', {});
    if isfield(cfg.rawData.run(runIdx), 'additionalRegressors_file')
        matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).multi_reg = {cfg.rawData.run(runIdx).additionalRegressors_file};
    end
    matlabbatch{moduleNr}.spm.stats.fmri_spec.sess(runIdx).hpf = cfg.GLM.HPF_perRun(runIdx); % high pass filter

end

matlabbatch{moduleNr}.spm.stats.fmri_spec.dir = {cfg.GLM.dataDir};
matlabbatch{moduleNr}.spm.stats.fmri_spec.timing.units = 'secs';
matlabbatch{moduleNr}.spm.stats.fmri_spec.timing.RT = cfg.GLM.TR;
matlabbatch{moduleNr}.spm.stats.fmri_spec.timing.fmri_t = cfg.GLM.microtimeResolution; % microtime resolution
matlabbatch{moduleNr}.spm.stats.fmri_spec.timing.fmri_t0 = cfg.GLM.microtimeOnset; % microtime onset
matlabbatch{moduleNr}.spm.stats.fmri_spec.fact = struct('name', {}, 'levels', {});
matlabbatch{moduleNr}.spm.stats.fmri_spec.bases.hrf.derivs = cfg.GLM.derivatives; % [0 0] no derivatives, [1 1] time and dispersion derivatives, [1 0]  time derivatives
matlabbatch{moduleNr}.spm.stats.fmri_spec.volt = 1;
matlabbatch{moduleNr}.spm.stats.fmri_spec.global = 'None';
matlabbatch{moduleNr}.spm.stats.fmri_spec.mthresh = cfg.GLM.maskingThreshold; % masking threshold
matlabbatch{moduleNr}.spm.stats.fmri_spec.mask = {''};
matlabbatch{moduleNr}.spm.stats.fmri_spec.cvi = 'AR(1)';


%% RUN SPM JOB AND REPORT DESIGN

% Create SPM figure for showing the design matrix (invisible)
% During SPM job execution, the design matrix is only shown if this figure exists (with Tag 'Graphics')
if isfield(cfg.GLM, 'dispDesignMatrix') && cfg.GLM.dispDesignMatrix==1
    hfig_designMatrix = spm_figure('Create', 'Graphics', 'Graphics', 'off');
end

% run SPM jobs
spm_jobman('initcfg');
spm_jobman('run', matlabbatch);

% display design matrix
if isfield(cfg.GLM, 'dispDesignMatrix') && cfg.GLM.dispDesignMatrix==1
    set(hfig_designMatrix, 'Visible', 'on');
    drawnow
end


