function updatePathData(handle_textbox, path)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% Display pathname or number of selected files/folders in textbox hande.
% Shorten pathname if necessary.

% do not show text during update process
set(handle_textbox, 'Visible', 'off');

pixelBuffer = 10; % Setting; pathname display will be shortened, if path-text length plus this buffer (number of pixels) is bigger than textbox-size

% prepare path-string for visualization in GUI textbox
if length(path)==1
    
    % only one file/dir has been selected
    
    visStr = path{1};
    
    set(handle_textbox, 'String', visStr);
    
    textboxPos = get(handle_textbox, 'Position');
    visStrExtent = get(handle_textbox, 'Extent');
    
    while visStrExtent(3)+pixelBuffer > textboxPos(3)
        visStr = visStr(2:end);
        set(handle_textbox, 'String', ['...' visStr]);
        visStrExtent = get(handle_textbox, 'Extent');
    end

else
    
    % multiple files have been selected, so just visualize number of files
    set(handle_textbox, 'String', [num2str(length(path)) ' files selected.']);
    
end

% visualize text again
set(handle_textbox, 'Visible', 'on');

% save on UserData
set(handle_textbox, 'UserData', path);

drawnow